CREATE TABLE JOB (  
	JOB_ID     INT                         NOT NULL ,  
	JOB_CLASS       VARCHAR(255)         NOT NULL,  
	NICKNAME        VARCHAR(50)         NOT NULL,  
	PICKUP_BUFFER_MINUTES	SMALLINT 						NOT NULL ,
	RECOVERY_TYPE   VARCHAR(20)             NOT NULL ,  
	MIN_EXECUTION_DURATION VARCHAR(15) 			NULL,
	MAX_EXECUTION_DURATION VARCHAR(15) 			NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL, 
	CHECK RECOVERY_TYPE IN ('NONE','ALL','LAST','CONFLICTED'),
	PRIMARY KEY  (JOB_ID)  
) ;

CREATE UNIQUE INDEX JOB_NICKNAME_UK ON JOB (NICKNAME);

CREATE TABLE JOB_STATE (
	JOB_STATE_ID 	INT 						NOT NULL ,
	JOB_ID     		INT                     	NOT NULL,  
	SCHEDULE        VARCHAR(255) 				NULL,  
	JOB_STATUS      VARCHAR(50)     NOT NULL,  
	EFFECTIVE_DATE	TIMESTAMP								NOT NULL,
	END_DATE		TIMESTAMP								NOT NULL,
	REVISION            INT                 	NOT NULL ,  
	CREATED_DATE        TIMESTAMP                        	NOT NULL,  
	CREATED_BY        VARCHAR(50)     	NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                          	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)    	NOT NULL,  
	CHECK JOB_STATUS IN ('DISABLED','ENABLED','CHAIN ACTIVE','AD HOC ACTIVE','UNSCHEDULED ACTIVE'),
	PRIMARY KEY  (JOB_STATE_ID)  
) ;

CREATE INDEX JOB_STATE_JOB_ID_NU ON JOB_STATE (JOB_ID);
ALTER TABLE JOB_STATE ADD CONSTRAINT JOB_STATE_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE JOB_RUNNING_HOST (
	JOB_RUNNING_HOST_ID		INT 						NOT NULL ,
	JOB_ID					INT 						NOT NULL,
	RUNNING_HOST		VARCHAR(50)									NOT NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_RUNNING_HOST_ID)  
) ;

CREATE INDEX JOB_RUNNING_HOST_NU ON JOB_RUNNING_HOST (JOB_ID);
CREATE UNIQUE INDEX JOB_RUNNING_HOST_HOST_UK ON JOB_RUNNING_HOST (JOB_ID, RUNNING_HOST);
ALTER TABLE JOB_RUNNING_HOST ADD CONSTRAINT JOB_RUNNING_HOST_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
    
CREATE TABLE JOB_CONFIGURATION (
	JOB_CONFIGURATION_ID    INT                         NOT NULL ,
	JOB_ID 			 		INT  						NOT NULL,  
	PARAMETER_NAME			VARCHAR(50)								NOT NULL,
	PARAMETER_TYPE			VARCHAR(255)							NOT NULL,
	VALUE					CLOB									NOT NULL,
	REVISION           		INT                         NOT NULL ,  
	CREATED_DATE        	TIMESTAMP                                NOT NULL,  
	CREATED_BY        		VARCHAR(50)            NOT NULL,  
	LAST_UPDATED_DATE 		TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY    		VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_CONFIGURATION_ID)  
) ;
CREATE INDEX JOB_CONFIGURATION_JOB_ID_NU ON JOB_CONFIGURATION (JOB_ID);
ALTER TABLE JOB_CONFIGURATION ADD CONSTRAINT JOB_CONFIGURATION_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
   
CREATE TABLE JOB_HISTORY (  
	JOB_HISTORY_ID     	INT                       	NOT NULL ,  
	JOB_ID 			  	INT  						NOT NULL,  
	STATUS      		VARCHAR(50)     NOT NULL,  
	SCHEDULED_TIME		TIMESTAMP            					NOT NULL,  
	RUNNING_HOST		VARCHAR(50)									NULL,
	PICKUP_TIME			TIMESTAMP            				    	NULL,  
	HEARTBEAT_TIME		TIMESTAMP            				    	NULL,  
	END_TIME		TIMESTAMP            				    		NULL,  
	AD_HOC		BOOLEAN                                 			NOT NULL ,
	RESUBMISSION		BOOLEAN                                 	NOT NULL , 
	CHAINED_FROM_ID 	INT 				NULL,  
	RESUBMITTED_TO_ID 	INT 				NULL,  
	SUBMISSION_MODE		VARCHAR(20)     NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            	NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	CHECK STATUS IN ('READY','RUNNING','COMPLETED','FAILED','DIED','CONFLICTED','MISSED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONFLICT MISSED'),
	CHECK SUBMISSION_MODE IN ('CONFLICTED_RECOVERY', 'CHAIN_SUBMISSION'),
	PRIMARY KEY  (JOB_HISTORY_ID)  
) ;
    
CREATE INDEX JOB_HISTORY_STATUS_NU ON JOB_HISTORY (STATUS);
CREATE INDEX JOB_HISTORY_RUNNING_HOST_NU ON JOB_HISTORY (RUNNING_HOST);
CREATE INDEX JOB_HISTORY_JOB_ID_NU ON JOB_HISTORY (JOB_ID);
CREATE INDEX JOB_HISTORY_CHAINED_FROM_JOB_HISTORY_ID_NU ON JOB_HISTORY (CHAINED_FROM_ID);
CREATE INDEX JOB_HISTORY_RESUBMITTED_TO_JOB_HISTORY_ID_NU ON JOB_HISTORY (RESUBMITTED_TO_ID);
CREATE UNIQUE INDEX JOB_HISTORY_SCHEDULED_JOB_UK ON JOB_HISTORY (JOB_ID, SCHEDULED_TIME);
CREATE INDEX JOB_HISTORY_SCHEDULED_TIME_NU ON JOB_HISTORY (SCHEDULED_TIME);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_CHAINED_FROM_FK FOREIGN KEY (CHAINED_FROM_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_RESUBMITTED_TO_FK FOREIGN KEY (RESUBMITTED_TO_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_ERROR (  
	JOB_HISTORY_ERROR_ID    INT                       NOT NULL ,
	JOB_HISTORY_ID     		INT                       NOT NULL,  
	EXCEPTION_CLASS			VARCHAR(300)						NOT NULL,
	EXCEPTION_MESSAGE		VARCHAR(255)						NOT NULL,
	ERROR					CLOB							NOT NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_HISTORY_ERROR_ID)  
) ;

CREATE UNIQUE INDEX JOB_HISTORY_ERROR_JOB_HISTORY_ID_UK_01 ON JOB_HISTORY_ERROR (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_ERROR ADD CONSTRAINT JOB_HISTORY_ERROR_JOB_HISTORY_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_INTERRUPT (  
	JOB_HISTORY_INTERRUPT_ID    INT                       NOT NULL ,
	JOB_HISTORY_ID     		INT                       NOT NULL,
	REQUESTER        VARCHAR(50)            NOT NULL,  
	REQUEST_TIME        TIMESTAMP                                NOT NULL,
	INTERRUPT_TIME        TIMESTAMP                                NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_HISTORY_INTERRUPT_ID)  
) ;

CREATE UNIQUE INDEX JOB_HISTORY_INTERRUPT_JOB_HISTORY_ID_UK_01 ON JOB_HISTORY_INTERRUPT (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_INTERRUPT ADD CONSTRAINT JOB_HISTORY_INTERRUPT_JOB_HISTORY_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_RESULT (  
	JOB_HISTORY_RESULT_ID   INT                     NOT NULL ,
	JOB_HISTORY_ID     		INT                     NOT NULL,  
	NAME					VARCHAR(255)						NOT NULL,
	VALUE					MEDIUMTEXT							    NULL,
	VALUE_TYPE				VARCHAR(255)						NOT NULL,
	REVISION            	INT                     NOT NULL ,  
	CREATED_DATE        	TIMESTAMP                            NOT NULL,  
	CREATED_BY        		VARCHAR(50)       NOT NULL,  
	LAST_UPDATED_DATE 		TIMESTAMP                            NOT NULL,  
	LAST_UPDATED_BY    		VARCHAR(50)       NOT NULL,  
	PRIMARY KEY  (JOB_HISTORY_RESULT_ID)  
) ;

ALTER TABLE JOB_HISTORY_RESULT ADD CONSTRAINT JOB_HISTORY_RESULT_JOB_HISTORY_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);


CREATE TABLE JOB_CHAIN (  
	JOB_CHAIN_ID     	INT                         NOT NULL ,  
	SOURCE_JOB_ID		INT  						NOT NULL,  
	CHAINED_TO_JOB_ID	INT  						NOT NULL,  
	CHAINING_ENABLED		BOOLEAN					NOT NULL ,  
	CHAINING_SCHEDULE        VARCHAR(255) 				NULL,  
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                   NOT NULL,  
	CREATED_BY        VARCHAR(50)    	      		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                  	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)    	      		NOT NULL,  
	PRIMARY KEY  (JOB_CHAIN_ID)  
) ;
    
CREATE INDEX JOB_CHAIN_SOURCE_JOB_ID_NU ON JOB_CHAIN (SOURCE_JOB_ID);
CREATE INDEX JOB_CHAIN_CHAINED_TO_JOB_ID_NU ON JOB_CHAIN (CHAINED_TO_JOB_ID);
CREATE INDEX JOB_CHAIN_BOTH_JOB_IDS_NU ON JOB_CHAIN (SOURCE_JOB_ID, CHAINED_TO_JOB_ID);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_SOURCE_JOB_FK FOREIGN KEY (SOURCE_JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_CHAINED_TO_FK FOREIGN KEY (CHAINED_TO_JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE JOB_CHAIN_MODE (  
	JOB_CHAIN_MODE_ID   INT                     	NOT NULL ,  
	JOB_CHAIN_ID     	INT                         NOT NULL,  
	STATUS      		VARCHAR(50)     			NOT NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                   NOT NULL,  
	CREATED_BY        VARCHAR(50)    	      NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)    	      NOT NULL,  
	CONSTRAINT JB_CH_MD_CHK CHECK STATUS IN ('FAILED','DIED','KILLED','MISSED','COMPLETED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONDITIONAL'),
	PRIMARY KEY  (JOB_CHAIN_MODE_ID)  
);
    
CREATE INDEX JOB_CHAIN_MODE_JOB_CHAIN_ID_NU ON JOB_CHAIN_MODE (JOB_CHAIN_ID);
ALTER TABLE JOB_CHAIN_MODE ADD CONSTRAINT JOB_CHAIN_MODE_JOB_CHAIN_ID_FK FOREIGN KEY (JOB_CHAIN_ID) REFERENCES JOB_CHAIN (JOB_CHAIN_ID);

CREATE TABLE JOB_CHAIN_MODE_CONDITION (
	JOB_CHAIN_MODE_CONDITION_ID		INT				NOT NULL,
	JOB_CHAIN_MODE_ID				INT				NOT NULL,
	CONTEXT_KEY						VARCHAR(255)	NOT NULL,
	OPERATOR						VARCHAR(30)		NOT NULL,
	VALUE							MEDIUMTEXT		    NULL,
	REVISION            			INT             NOT NULL,  
	CREATED_DATE        			TIMESTAMP       NOT NULL,  
	CREATED_BY        				VARCHAR(50)    	NOT NULL,  
	LAST_UPDATED_DATE 				TIMESTAMP       NOT NULL,  
	LAST_UPDATED_BY    				VARCHAR(50)    	NOT NULL,
	CHECK OPERATOR IN ('EQUALS','NOT_EQUALS','IN','NOT_IN','GREATER_THAN','LESS_THAN','GREATER_THAN_OR_EQUAL','LESS_THAN_OR_EQUAL','STARTS_WITH','ENDS_WITH','CONTAINS','EXISTS','NOT_EXISTS','REGEXP'),
	PRIMARY KEY  (JOB_CHAIN_MODE_CONDITION_ID)    
);

CREATE INDEX JOB_CHAIN_MODE_CONDITION_JOB_CHAIN_MODE_ID_NU ON JOB_CHAIN_MODE_CONDITION (JOB_CHAIN_MODE_ID);
ALTER TABLE JOB_CHAIN_MODE_CONDITION ADD CONSTRAINT JOB_CHAIN_MODE_CONDITION_FK FOREIGN KEY (JOB_CHAIN_MODE_ID) REFERENCES JOB_CHAIN_MODE (JOB_CHAIN_MODE_ID);


CREATE TABLE JOB_HISTORY_CHAIN (  
	JOB_HISTORY_CHAIN_ID    INT                     NOT NULL ,
	JOB_HISTORY_ID     		INT                     NOT NULL,  
	JOB_CHAIN_ID			INT 					NOT NULL,
	CHAIN_TRIGGER			BOOLEAN									NOT NULL,
	CHAIN_DETAIL			VARCHAR(255)							NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            	NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_HISTORY_CHAIN_ID)  
) ;

CREATE INDEX JOB_HISTORY_CHAIN_JOB_HISTORY_ID_NU ON JOB_HISTORY_CHAIN (JOB_HISTORY_ID, CHAIN_TRIGGER);
CREATE INDEX JOB_HISTORY_CHAIN_JOB_CHAIN_ID_NU ON JOB_HISTORY_CHAIN (JOB_CHAIN_ID);
CREATE UNIQUE INDEX JOB_HISTORY_JOB_HISTORY_CHAIN_UK_01 ON JOB_HISTORY_CHAIN (JOB_HISTORY_ID, JOB_CHAIN_ID);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HISTORY_CHAIN_JOB_HISTORY_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HISTORY_CHAIN_JOB_CHAIN_FK FOREIGN KEY (JOB_CHAIN_ID) REFERENCES JOB_CHAIN (JOB_CHAIN_ID);

CREATE TABLE JOB_CONFLICT_CONFIG (  
	JOB_CONFLICT_CONFIG_ID     	INT                 NOT NULL ,  
	PRIORITY_JOB_ID				INT  				NOT NULL,  
	YIELDING_JOB_ID				INT  				NOT NULL,
	GROUP_NUMBER				INT  				NOT NULL,
	ORDINAL 					INT  				NOT NULL,
	REVISION            INT                         NOT NULL ,  
	CREATED_DATE        TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)            	NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                               	NOT NULL,  
	LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,  
	PRIMARY KEY  (JOB_CONFLICT_CONFIG_ID)  
);
    
CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_01 ON JOB_CONFLICT_CONFIG (PRIORITY_JOB_ID);
CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_02 ON JOB_CONFLICT_CONFIG (YIELDING_JOB_ID);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_CONFIG_PRIORITY_JOB_FK FOREIGN KEY (PRIORITY_JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_CONFIG_YIELDING_JOB_FK FOREIGN KEY (YIELDING_JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE LICENCE (  
	LICENCE_ID        INT       NOT NULL ,  
	LICENCE_TYPE      VARCHAR(10) NOT NULL,  
	ISSUE_DATE        TIMESTAMP              NOT NULL,
	EXPIRY_DATE       TIMESTAMP              NOT NULL,
	LICENCE_KEY       VARCHAR(255) 			NOT NULL,
	PROXY_RENEWAL_TIME      TIMESTAMP                                    NULL,
	PROXY_LEASE_EXPIRY_TIME TIMESTAMP                                    NULL,
	PROXY_RENEWAL_IP        VARCHAR(255)							  	NULL,
	PROXY_RENEWAL_TOKEN     VARCHAR(255)							  	NULL,
	PURCHASE_DETAIL_MESSAGE CLOB										NULL,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL,  
	CHECK LICENCE_TYPE IN ('TRIAL','FLOAT'),
	PRIMARY KEY  (LICENCE_ID)  
) ;

CREATE UNIQUE INDEX LICENCE_KEY_UK ON LICENCE (LICENCE_KEY);

CREATE TABLE KEY_INSTANCE (  
	KEY_INSTANCE_ID   INT                         NOT NULL ,
	LICENCE_ID        INT                         NOT NULL,  
	RENEWAL_TIME      TIMESTAMP                                    NULL,
	LEASE_EXPIRY_TIME TIMESTAMP                                    NULL,
	RENEWAL_IP        VARCHAR(255)							  	  NULL,
	RENEWAL_TOKEN     VARCHAR(255)							  	  NULL,
	EXPIRY_TIME		  TIMESTAMP                                	  NULL,
	REVISION          INT                         NOT NULL ,  
	CREATED_DATE      TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)    					      NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)   					      NOT NULL,  
	PRIMARY KEY  (KEY_INSTANCE_ID)  
) ;

CREATE INDEX KEY_INSTANCE_LICENCE_ID_NU ON KEY_INSTANCE (LICENCE_ID);
ALTER TABLE KEY_INSTANCE ADD CONSTRAINT KEY_INSTANCE_LICENCE_FK FOREIGN KEY (LICENCE_ID) REFERENCES LICENCE (LICENCE_ID);

CREATE TABLE TRIAL_LICENCE (
	TRIAL_LICENCE_ID 	INT 				NOT NULL ,
	IP_ADDRESS			VARCHAR(255)					NOT NULL,
	LICENCE_ID 			INT 					NULL ,
	REVISION          INT                         NOT NULL ,  
	CREATED_DATE      TIMESTAMP                                NOT NULL,  
	CREATED_BY        VARCHAR(50)    					      NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)   					      NOT NULL,  
	PRIMARY KEY  (TRIAL_LICENCE_ID)  
) ;
CREATE INDEX TRIAL_LICENCE_NU ON TRIAL_LICENCE (IP_ADDRESS);
ALTER TABLE TRIAL_LICENCE ADD CONSTRAINT TRIAL_LICENCE_LICENCE_FK FOREIGN KEY (LICENCE_ID) REFERENCES LICENCE (LICENCE_ID);

CREATE TABLE EVENT_LOG (
	EVENT_LOG_ID	INT 				NOT NULL ,
	EVENT_LEVEL			VARCHAR(10)				NOT NULL,
	EVENT_TIME		TIMESTAMP				NOT NULL,
	HOST			VARCHAR(100)			NOT NULL,
	CATEGORY		VARCHAR(50)				NOT NULL,
	SUMMARY			VARCHAR(255)			NOT NULL,
	MESSAGE			CLOB				    NULL,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	PRIMARY KEY  (EVENT_LOG_ID)  
) ;

CREATE INDEX KEY_LOG_NU_01 ON EVENT_LOG (CATEGORY);
CREATE INDEX KEY_LOG_NU_02 ON EVENT_LOG (HOST);
CREATE INDEX KEY_LOG_NU_03 ON EVENT_LOG (EVENT_TIME, EVENT_LEVEL, CATEGORY, HOST);

CREATE TABLE SEMAPHORE (
	SEMAPHORE_ID	INT 					NOT NULL ,
	NAME			VARCHAR(50)							NOT NULL,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	PRIMARY KEY  (SEMAPHORE_ID)  
) ;

CREATE UNIQUE INDEX KEY_SEMAPHORE_UK_01 ON SEMAPHORE (NAME);

CREATE TABLE OPERATIONS_PARAMETER (
	OPERATIONS_PARAMETER_ID	INT 					NOT NULL ,
	NAME				VARCHAR(100)							NOT NULL,
	VALUE				VARCHAR(255)						NOT NULL,
	TYPE				VARCHAR(100)							NULL,
	CATEGORY			VARCHAR(100)						NOT NULL,
	DESCRIPTION			VARCHAR(500)							NULL,
	EDITABLE			BOOLEAN									NOT NULL,
	UPDATE_TIME			TIMESTAMP							NOT NULL,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	PRIMARY KEY  (OPERATIONS_PARAMETER_ID)  
);

CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_01 ON OPERATIONS_PARAMETER (NAME);
CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_02 ON OPERATIONS_PARAMETER (CATEGORY);


CREATE TABLE EVENT_SUBSCRIBER (
	EVENT_SUBSCRIBER_ID	INT 	NOT NULL ,
	ADDRESS			  VARCHAR(255)			NOT NULL,
	ADDRESS_TYPE      VARCHAR(10) 		NOT NULL, 
	ACTIVE 			  BOOLEAN 					NOT NULL ,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	CHECK ADDRESS_TYPE IN ('EMAIL'),
	PRIMARY KEY  (EVENT_SUBSCRIBER_ID)  
) ;

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIBER_UK_01 ON EVENT_SUBSCRIBER (ADDRESS, ADDRESS_TYPE);

CREATE TABLE EVENT_SUBSCRIPTION (
	EVENT_SUBSCRIPTION_ID INT   NOT NULL ,
	EVENT_SUBSCRIBER_ID	INT     NOT NULL ,
	CATEGORY		  VARCHAR(255)			NOT NULL,
	ENTITY_ID		  INT 			NULL,
	MIN_LEVEL		  VARCHAR(255)			NOT NULL,
	ACTIVE 			  BOOLEAN 					NOT NULL ,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	PRIMARY KEY  (EVENT_SUBSCRIPTION_ID)  
) ;

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIPTION_UK_01 ON EVENT_SUBSCRIPTION (EVENT_SUBSCRIBER_ID, CATEGORY, ENTITY_ID);
CREATE INDEX KEY_EVENT_SUBSCRIBER_NU_01 ON EVENT_SUBSCRIPTION (CATEGORY);
ALTER TABLE EVENT_SUBSCRIPTION ADD CONSTRAINT EVENT_SUBCRIPTION_SUBSCRIBER_FK FOREIGN KEY (EVENT_SUBSCRIBER_ID) REFERENCES EVENT_SUBSCRIBER (EVENT_SUBSCRIBER_ID);

CREATE TABLE NOTIFICATION (
	NOTIFICATION_ID	INT 		NOT NULL ,
	EVENT_LOG_ID	INT 		NOT NULL,
	EVENT_SUBSCRIBER_ID	INT 	NOT NULL,
	REVISION          INT       NOT NULL ,  
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        VARCHAR(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   VARCHAR(50)    		NOT NULL, 
	PRIMARY KEY  (NOTIFICATION_ID)  
) ;

CREATE UNIQUE INDEX KEY_NOTIFICATION_UK_01 ON NOTIFICATION (EVENT_LOG_ID, EVENT_SUBSCRIBER_ID);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_SUBSCRIBER_FK FOREIGN KEY (EVENT_SUBSCRIBER_ID) REFERENCES EVENT_SUBSCRIBER (EVENT_SUBSCRIBER_ID);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_LOG_FK FOREIGN KEY (EVENT_LOG_ID) REFERENCES EVENT_LOG (EVENT_LOG_ID);
