CREATE TABLE AUDIT_TRAIL (
  AUDIT_TRAIL_ID     INT                         NOT NULL ,
  TABLE_NAME        VARCHAR(50)         NOT NULL,
  COLUMN_NAME        VARCHAR(50)         NOT NULL,
  ROW_ID            INT                          NOT NULL,
  ACTION            CHAR(1)     NOT NULL,
  VALUE           VARCHAR(255)             NULL,
  ENTITY_NAME        VARCHAR(50)         NOT NULL,
  AUDIT_DATE         TIMESTAMP                                NOT NULL,
  REVISION            INT                         NOT NULL ,
  CREATED_DATE        TIMESTAMP                                NOT NULL,
  CREATED_BY        VARCHAR(50)            NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,
  CONSTRAINT CHECK ACTION IN ('I','U','D','C'),
  PRIMARY KEY  (AUDIT_TRAIL_ID)
) ;

ALTER TABLE AUDIT_TRAIL ADD ;

CREATE INDEX NU_AUDIT_TRAIL_01 ON AUDIT_TRAIL (TABLE_NAME, COLUMN_NAME);
CREATE INDEX NU_AUDIT_TRAIL_02 ON AUDIT_TRAIL (TABLE_NAME, ROW_ID);


CREATE TABLE ATTACHMENT (
  ATTACHMENT_ID     INT                         NOT NULL ,
  OWNER_ID            INT                             NULL,
  NAME                VARCHAR(500)         NOT NULL,
  CONTENT_TYPE        VARCHAR(100)         NOT NULL,
  ADDED_DATE        TIMESTAMP                                NOT NULL,
  CONFIRMED            BOOLEAN                                        NOT NULL,
  CONTENT            BLOB                                    NULL,
  REVISION            INT                         NOT NULL ,
  CREATED_DATE        TIMESTAMP                                NOT NULL,
  CREATED_BY        VARCHAR(50)            NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,
  PRIMARY KEY  (ATTACHMENT_ID)
) ;

CREATE TABLE DOMAIN_ELEMENT (
  DOMAIN_ELEMENT_ID INT                         NOT NULL ,
  DOMAIN            VARCHAR(50)         NOT NULL,
  REF_CODE            VARCHAR(50)         NOT NULL,
  DESCRIPTION        VARCHAR(200)         NOT NULL,
  REVISION            INT                         NOT NULL ,
  CREATED_DATE        TIMESTAMP                                NOT NULL,
  CREATED_BY        VARCHAR(50)            NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,
  PRIMARY KEY  (DOMAIN_ELEMENT_ID)
) ;
  
CREATE UNIQUE INDEX UK_DOMAIN_ELEMENT_01 ON DOMAIN_ELEMENT (DOMAIN, REF_CODE);


CREATE TABLE SEQUENCE_MANAGER (
  NAME                 VARCHAR(50)          NOT NULL,
  VALUE             INT                          NOT NULL ,
  REVISION            INT                         NOT NULL,
  CREATED_DATE        TIMESTAMP                                NOT NULL,
  CREATED_BY        VARCHAR(50)            NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)            NOT NULL,
  PRIMARY KEY  (NAME)
) ;


CREATE TABLE SUITE_USER (
 SUITE_USER_ID        INT                     NOT NULL ,
 USER_NAME            VARCHAR(50)     NOT NULL,
 PASSWORD            VARCHAR(100)     NOT NULL,
 FIRST_NAME            VARCHAR(50)     NULL,
 LAST_NAME            VARCHAR(50)      NULL,
 EMAIL                VARCHAR(50)      NULL,
 ACTIVE        		   BOOLEAN                               NOT NULL default true, 
 REVISION            INT                     NOT NULL,
 CREATED_DATE        TIMESTAMP                            NOT NULL,
 CREATED_BY            VARCHAR(50)        NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)        NOT NULL,
 PRIMARY KEY (SUITE_USER_ID)
) ;

CREATE UNIQUE INDEX UK_USER_USER_NAME ON SUITE_USER (USER_NAME);

CREATE TABLE USER_COOKIE (
 USER_COOKIE_ID            INT   NOT NULL ,
 USER_NAME            VARCHAR(50)     NOT NULL,
 COOKIE            	  VARCHAR(255)     NOT NULL, 
 USER_STATE           TEXT     NOT NULL, 
 REVISION            INT                     NOT NULL,
 CREATED_DATE        TIMESTAMP                            NOT NULL,
 CREATED_BY            VARCHAR(50)        NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)        NOT NULL,
 PRIMARY KEY (USER_COOKIE_ID)
) ;

CREATE UNIQUE INDEX UK_USER_COOKIE_USER_NAME ON USER_COOKIE (USER_NAME);
CREATE UNIQUE INDEX UK_USER_COOKIE_COOKIE ON USER_COOKIE (COOKIE);


CREATE TABLE ROLE (
 ROLE_ID            INT                     NOT NULL ,
 ROLE_NAME            VARCHAR(50)     NOT NULL,
 REVISION            INT                     NOT NULL ,
 CREATED_DATE        TIMESTAMP                            NOT NULL,
 CREATED_BY            VARCHAR(50)        NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)        NOT NULL,
 PRIMARY KEY (ROLE_ID)
) ;

CREATE UNIQUE INDEX UK_ROLE_NAME ON ROLE (ROLE_NAME);

CREATE TABLE USER_ROLE (
 USER_ROLE_ID        INT                     NOT NULL ,
 SUITE_USER_ID		INT                     NOT NULL,
 ROLE_ID            INT                      NOT NULL,
 REVISION            INT                     NOT NULL ,
 CREATED_DATE        TIMESTAMP                            NOT NULL,
 CREATED_BY            VARCHAR(50)        NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)        NOT NULL,
 PRIMARY KEY (USER_ROLE_ID),
 FOREIGN KEY (SUITE_USER_ID) REFERENCES SUITE_USER (SUITE_USER_ID),
 FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID) 
) ;

CREATE UNIQUE INDEX UK_USER_ROLE ON USER_ROLE (SUITE_USER_ID, ROLE_ID);


CREATE TABLE ROLE_ACTION_MAP (
 ROLE_ACTION_MAP_ID                 INT                     NOT NULL ,
 ROLE_ID                            INT                     NOT NULL,
 SERVLET_CLASS                        VARCHAR(100)    NOT NULL,
 ACTION_NAME                        VARCHAR(50)        NOT NULL,
 REVISION                            INT                     NOT NULL ,
 CREATED_DATE                        TIMESTAMP                            NOT NULL,
 CREATED_BY                            VARCHAR(50)        NOT NULL,
 LAST_UPDATED_DATE                  TIMESTAMP                            NOT NULL,
 LAST_UPDATED_BY                    VARCHAR(50)        NOT NULL,
 FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID),
 PRIMARY KEY (ROLE_ACTION_MAP_ID)
) ;

CREATE UNIQUE INDEX UK_ROLE_ACTION_MAP_01 ON ROLE_ACTION_MAP (ROLE_ID,SERVLET_CLASS,ACTION_NAME);
