CREATE TABLE AUDIT_TRAIL (
  AUDIT_TRAIL_ID     INT(10) UNSIGNED                        NOT NULL AUTO_INCREMENT,
  TABLE_NAME        VARCHAR(50) character set utf8        NOT NULL,
  COLUMN_NAME        VARCHAR(50) character set utf8        NOT NULL,
  ROW_ID            INT(10) UNSIGNED                         NOT NULL,
  ACTION            ENUM('I','U','D','C') character set utf8    NOT NULL,
  VALUE           VARCHAR(255) character set utf8            NULL,
  ENTITY_NAME        VARCHAR(50) character set utf8        NOT NULL,
  AUDIT_DATE         DATETIME                                NOT NULL,
  REVISION            INT(10) UNSIGNED                        NOT NULL default '0',
  CREATED_DATE        DATETIME                                NOT NULL,
  CREATED_BY        VARCHAR(50)    character set utf8        NOT NULL,
  LAST_UPDATED_DATE DATETIME                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)    character set utf8        NOT NULL,
  PRIMARY KEY  (AUDIT_TRAIL_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT='JDK generic audit trail table';

CREATE INDEX NU_AUDIT_TRAIL_01 ON AUDIT_TRAIL (TABLE_NAME, COLUMN_NAME);
CREATE INDEX NU_AUDIT_TRAIL_02 ON AUDIT_TRAIL (TABLE_NAME, ROW_ID);


CREATE TABLE ATTACHMENT (
  `ATTACHMENT_ID`     INT(10) UNSIGNED                        NOT NULL AUTO_INCREMENT,
  `OWNER_ID`            INT(10) UNSIGNED                            NULL,
  `NAME`                VARCHAR(500) character set utf8        NOT NULL,
  `CONTENT_TYPE`        VARCHAR(100) character set utf8        NOT NULL,
  `ADDED_DATE`        DATETIME                                NOT NULL,
  `CONFIRMED`            BIT                                        NOT NULL,
  `CONTENT`            LONGBLOB                                    NULL,
  REVISION            INT(10) UNSIGNED                        NOT NULL ,
  CREATED_DATE        DATETIME                                NOT NULL,
  CREATED_BY        VARCHAR(50)    character set utf8        NOT NULL,
  LAST_UPDATED_DATE DATETIME                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)    character set utf8        NOT NULL,
  PRIMARY KEY  (`ATTACHMENT_ID`)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE TABLE DOMAIN_ELEMENT (
  DOMAIN_ELEMENT_ID INT(10) UNSIGNED                        NOT NULL AUTO_INCREMENT,
  DOMAIN            VARCHAR(50) character set utf8        NOT NULL,
  REF_CODE            VARCHAR(50) character set utf8        NOT NULL,
  DESCRIPTION        VARCHAR(200) character set utf8        NOT NULL,
  REVISION            INT(10) UNSIGNED                        NOT NULL ,
  CREATED_DATE        DATETIME                                NOT NULL,
  CREATED_BY        VARCHAR(50)    character set utf8        NOT NULL,
  LAST_UPDATED_DATE DATETIME                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)    character set utf8        NOT NULL,
  PRIMARY KEY  (DOMAIN_ELEMENT_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';
  
CREATE UNIQUE INDEX UK_DOMAIN_ELEMENT_01 ON DOMAIN_ELEMENT (DOMAIN, REF_CODE);


CREATE TABLE SEQUENCE_MANAGER (
  NAME                 VARCHAR(50) character set utf8         NOT NULL,
  VALUE             INT(10) UNSIGNED                         NOT NULL ,
  REVISION            INT(10) UNSIGNED                        NOT NULL,
  CREATED_DATE        DATETIME                                NOT NULL,
  CREATED_BY        VARCHAR(50)    character set utf8        NOT NULL,
  LAST_UPDATED_DATE DATETIME                                NOT NULL,
  LAST_UPDATED_BY    VARCHAR(50)    character set utf8        NOT NULL,
  PRIMARY KEY  (NAME)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT='table to manage common sequence functionality across db impl';


CREATE TABLE SUITE_USER (
 SUITE_USER_ID        INT(10) UNSIGNED                    NOT NULL AUTO_INCREMENT,
 USER_NAME            VARCHAR(50) character set utf8    NOT NULL,
 PASSWORD            VARCHAR(100) character set utf8    NOT NULL,
 FIRST_NAME            VARCHAR(50) character set utf8    NULL,
 LAST_NAME            VARCHAR(50)  character set utf8    NULL,
 EMAIL                VARCHAR(50) character set utf8     NULL,
 ACTIVE        		   BIT                               NOT NULL default true, 
 REVISION            INT(10) UNSIGNED                    NOT NULL,
 CREATED_DATE        DATETIME                            NOT NULL,
 CREATED_BY            VARCHAR(50)    character set utf8    NOT NULL,
 LAST_UPDATED_DATE  DATETIME                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)    character set utf8    NOT NULL,
 PRIMARY KEY (SUITE_USER_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE UNIQUE INDEX UK_USER_USER_NAME ON SUITE_USER (USER_NAME);

CREATE TABLE USER_COOKIE (
 USER_COOKIE_ID            INT(10) UNSIGNED  NOT NULL AUTO_INCREMENT,
 USER_NAME            VARCHAR(50) character set utf8    NOT NULL,
 COOKIE            	  VARCHAR(255) character set utf8    NOT NULL, 
 USER_STATE           TEXT character set utf8    NOT NULL, 
 REVISION            INT(10) UNSIGNED                    NOT NULL,
 CREATED_DATE        DATETIME                            NOT NULL,
 CREATED_BY            VARCHAR(50)    character set utf8    NOT NULL,
 LAST_UPDATED_DATE  DATETIME                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)    character set utf8    NOT NULL,
 PRIMARY KEY (USER_COOKIE_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE UNIQUE INDEX UK_USER_COOKIE_USER_NAME ON USER_COOKIE (USER_NAME);
CREATE UNIQUE INDEX UK_USER_COOKIE_COOKIE ON USER_COOKIE (COOKIE);


CREATE TABLE `ROLE` (
 ROLE_ID            INT(10) UNSIGNED                    NOT NULL AUTO_INCREMENT,
 ROLE_NAME            VARCHAR(50) character set utf8    NOT NULL,
 REVISION            INT(10) UNSIGNED                    NOT NULL ,
 CREATED_DATE        DATETIME                            NOT NULL,
 CREATED_BY            VARCHAR(50)    character set utf8    NOT NULL,
 LAST_UPDATED_DATE  DATETIME                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)    character set utf8    NOT NULL,
 PRIMARY KEY (ROLE_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE UNIQUE INDEX UK_ROLE_NAME ON `ROLE` (ROLE_NAME);

CREATE TABLE USER_ROLE (
 USER_ROLE_ID        INT(10) UNSIGNED                    NOT NULL AUTO_INCREMENT,
 SUITE_USER_ID		INT(10) UNSIGNED                    NOT NULL,
 ROLE_ID            INT(10) UNSIGNED                     NOT NULL,
 REVISION            INT(10) UNSIGNED                    NOT NULL ,
 CREATED_DATE        DATETIME                            NOT NULL,
 CREATED_BY            VARCHAR(50)    character set utf8    NOT NULL,
 LAST_UPDATED_DATE  DATETIME                            NOT NULL,
 LAST_UPDATED_BY    VARCHAR(50)    character set utf8    NOT NULL,
 PRIMARY KEY (USER_ROLE_ID),
 FOREIGN KEY (SUITE_USER_ID) REFERENCES SUITE_USER (SUITE_USER_ID),
 FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID) 
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE UNIQUE INDEX UK_USER_ROLE ON USER_ROLE (SUITE_USER_ID, ROLE_ID);


CREATE TABLE ROLE_ACTION_MAP (
 ROLE_ACTION_MAP_ID                 INT(10) UNSIGNED                    NOT NULL AUTO_INCREMENT,
 ROLE_ID                            INT(10) UNSIGNED                    NOT NULL,
 SERVLET_CLASS                        VARCHAR(100)character set utf8    NOT NULL,
 ACTION_NAME                        VARCHAR(50)character set utf8        NOT NULL,
 REVISION                            INT(10) UNSIGNED                    NOT NULL ,
 CREATED_DATE                        DATETIME                            NOT NULL,
 CREATED_BY                            VARCHAR(50)character set utf8        NOT NULL,
 LAST_UPDATED_DATE                  DATETIME                            NOT NULL,
 LAST_UPDATED_BY                    VARCHAR(50)character set utf8        NOT NULL,
 PRIMARY KEY (ROLE_ACTION_MAP_ID),
 FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID)
) Engine=InnoDB DEFAULT CHARSET=utf8 COLLATE utf8_bin  COMMENT = 'SUITE';

CREATE UNIQUE INDEX UK_ROLE_ACTION_MAP_01 ON ROLE_ACTION_MAP (ROLE_ID,SERVLET_CLASS,ACTION_NAME);
