CREATE TABLE JOB (  
	JOB_ID          NUMBER(10)       					PRIMARY KEY,  
	JOB_CLASS       NVARCHAR2(255)                      NOT NULL,  
	NICKNAME        NVARCHAR2(50)                       NOT NULL,  
	PICKUP_BUFFER_MINUTES	NUMBER(4) 	default '2'		NOT NULL,
	RECOVERY_TYPE   NVARCHAR2(20)  default 'NONE'       NOT NULL,  
	MIN_EXECUTION_DURATION NVARCHAR2(15)					NULL,
	MAX_EXECUTION_DURATION NVARCHAR2(15)					NULL,
	REVISION            NUMBER(10)     default '0'      NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                     NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                    NOT NULL,  
	CONSTRAINT CHK_JOB_RECOVERY_TYPE CHECK (RECOVERY_TYPE IN ('NONE','ALL','LAST','CONFLICTED'))
);

CREATE UNIQUE INDEX JOB_NICKNAME_UK ON JOB (NICKNAME);

CREATE TABLE JOB_STATE (
	JOB_STATE_ID 	NUMBER(10)						PRIMARY KEY,
	JOB_ID     		NUMBER(10)                    	NOT NULL,  
	SCHEDULE        NVARCHAR2(255)                      NULL,  
	JOB_STATUS      NVARCHAR2(50)                    NOT NULL,  
	EFFECTIVE_DATE	TIMESTAMP						NOT NULL,
	END_DATE		TIMESTAMP						NOT NULL,
	REVISION            NUMBER(10)   default '0'    NOT NULL,  
	CREATED_DATE        TIMESTAMP                   NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                 NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                     NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                NOT NULL,  
	CONSTRAINT CHK_STATE_STATUS CHECK (JOB_STATUS IN ('DISABLED','ENABLED','CHAIN ACTIVE','AD HOC ACTIVE','UNSCHEDULED ACTIVE'))  
);

CREATE INDEX JOB_STATE_JOB_ID_NU ON JOB_STATE (JOB_ID);
ALTER TABLE JOB_STATE ADD CONSTRAINT JOB_STATE_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE JOB_RUNNING_HOST (
	JOB_RUNNING_HOST_ID		NUMBER(10)						PRIMARY KEY,
	JOB_ID					NUMBER(10)						NOT NULL,
	RUNNING_HOST		NVARCHAR2(50)						NOT NULL,
	REVISION            NUMBER(10)  default '0'             NOT NULL,  
	CREATED_DATE        TIMESTAMP                           NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                         NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                             NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                        NOT NULL  
);

CREATE INDEX JOB_RUNNING_HOST_NU ON JOB_RUNNING_HOST (JOB_ID);
CREATE UNIQUE INDEX JOB_RUNNING_HOST_HOST_UK ON JOB_RUNNING_HOST (JOB_ID, RUNNING_HOST);
ALTER TABLE JOB_RUNNING_HOST ADD CONSTRAINT JOB_RUNNING_HOST_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
    
CREATE TABLE JOB_CONFIGURATION (
	JOB_CONFIGURATION_ID    NUMBER(10)                        PRIMARY KEY,
	JOB_ID 			 		NUMBER(10) 						  NOT NULL,  
	PARAMETER_NAME			NVARCHAR2(50)					  NOT NULL,
	PARAMETER_TYPE			NVARCHAR2(255)				      NOT NULL,
	VALUE					NCLOB							  NOT NULL,
	REVISION           		NUMBER(10) default '0'            NOT NULL ,  
	CREATED_DATE        	TIMESTAMP                         NOT NULL,  
	CREATED_BY        		NVARCHAR2(50)                     NOT NULL,  
	LAST_UPDATED_DATE 		TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    		NVARCHAR2(50)                     NOT NULL  
);

CREATE INDEX JOB_CONFIGURATION_JOB_ID_NU ON JOB_CONFIGURATION (JOB_ID);
ALTER TABLE JOB_CONFIGURATION ADD CONSTRAINT JOB_CONFIGURATION_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
   
CREATE TABLE JOB_HISTORY (  
	JOB_HISTORY_ID     	NUMBER(10)                      PRIMARY KEY,  
	JOB_ID 			  	NUMBER(10) 						NOT NULL,  
	STATUS      		NVARCHAR2(20)                   NOT NULL,  
	SCHEDULED_TIME		TIMESTAMP            			NOT NULL,  
	RUNNING_HOST		NVARCHAR2(50)						NULL,
	PICKUP_TIME			TIMESTAMP            				NULL,  
	HEARTBEAT_TIME		TIMESTAMP            				NULL,  
	END_TIME		TIMESTAMP            					NULL,  
	AD_HOC		NUMBER(1) default '0'					NOT NULL, 
	RESUBMISSION		NUMBER(1) default '0'			NOT NULL, 
	CHAINED_FROM_ID 	NUMBER(10)							NULL,  
	RESUBMITTED_TO_ID 	NUMBER(10)							NULL,  
	SUBMISSION_MODE		NVARCHAR2(20)                       NULL,
	REVISION            NUMBER(10) default '0'			NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                     NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                    NOT NULL,  
	CONSTRAINT CHK_JOB_HISTORY_STATUS CHECK (STATUS IS NULL OR STATUS IN ('READY','RUNNING','COMPLETED','FAILED','DIED','CONFLICTED','MISSED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONFLICT MISSED')),
	CONSTRAINT CHK_JOB_HISTORY_SUBM_MODE CHECK (SUBMISSION_MODE IS NULL OR SUBMISSION_MODE IN ('CONFLICTED_RECOVERY', 'CHAIN_SUBMISSION'))  
);
    
CREATE INDEX JOB_HISTORY_STATUS_NU ON JOB_HISTORY (STATUS);
CREATE INDEX JOB_HISTORY_RUNNING_HOST_NU ON JOB_HISTORY (RUNNING_HOST);
CREATE INDEX JOB_HISTORY_JOB_ID_NU ON JOB_HISTORY (JOB_ID);
CREATE INDEX JOB_HIST_CHAINED_FROM_NU ON JOB_HISTORY (CHAINED_FROM_ID);
CREATE INDEX JOB_HIST_RESUBMITTED_TO_NU ON JOB_HISTORY (RESUBMITTED_TO_ID);
CREATE UNIQUE INDEX JOB_HISTORY_SCHEDULED_JOB_UK ON JOB_HISTORY (JOB_ID, SCHEDULED_TIME);
CREATE INDEX JOB_HISTORY_SCHEDULED_TIME_NU ON JOB_HISTORY (SCHEDULED_TIME);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_JOB_FK FOREIGN KEY (JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_CHAINED_FROM_FK FOREIGN KEY (CHAINED_FROM_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_RESUBMITTED_TO_FK FOREIGN KEY (RESUBMITTED_TO_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_ERROR (  
	JOB_HISTORY_ERROR_ID    NUMBER(10)                      PRIMARY KEY,
	JOB_HISTORY_ID     		NUMBER(10)                      NOT NULL,  
	EXCEPTION_CLASS			NVARCHAR2(300)					NOT NULL,
	EXCEPTION_MESSAGE		NVARCHAR2(255)					NOT NULL,
	ERROR					NCLOB							NOT NULL,
	REVISION            NUMBER(10)  default '0'				NOT NULL,  
	CREATED_DATE        TIMESTAMP                           NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                         NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                             NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                        NOT NULL
);

CREATE UNIQUE INDEX JOB_HIST_ERROR_HIST_ID_UK_01 ON JOB_HISTORY_ERROR (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_ERROR ADD CONSTRAINT JOB_HIST_ERROR_JOB_HIST_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_INTERRUPT (  
	JOB_HISTORY_INTERRUPT_ID    NUMBER(10)                      PRIMARY KEY,
	JOB_HISTORY_ID     		NUMBER(10)                      NOT NULL,  
	REQUESTER         NVARCHAR2(50)                         NOT NULL,  
	REQUEST_TIME		TIMESTAMP							NOT NULL,
	INTERRUPT_TIME		TIMESTAMP							    NULL,
	REVISION            NUMBER(10)  default '0'				NOT NULL,  
	CREATED_DATE        TIMESTAMP                           NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                         NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                             NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                        NOT NULL
);

CREATE UNIQUE INDEX JOB_HIST_INTRP_HIST_ID_UK_01 ON JOB_HISTORY_INTERRUPT (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_INTERRUPT ADD CONSTRAINT JOB_HIST_INTRP_JOB_HIST_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);

CREATE TABLE JOB_HISTORY_RESULT (  
	JOB_HISTORY_RESULT_ID   NUMBER(10)                    PRIMARY KEY,
	JOB_HISTORY_ID     		NUMBER(10)                    NOT NULL,  
	NAME					NVARCHAR2(255)				  NOT NULL,
	VALUE					NVARCHAR2(2000)					  NULL,
	VALUE_TYPE				NVARCHAR2(255)				  NOT NULL,
	REVISION            	NUMBER(10) default '0'        NOT NULL,  
	CREATED_DATE        	TIMESTAMP             		  NOT NULL,  
	CREATED_BY        		NVARCHAR2(50)                 NOT NULL,  
	LAST_UPDATED_DATE 		TIMESTAMP                     NOT NULL,  
	LAST_UPDATED_BY    		NVARCHAR2(50)                 NOT NULL
);

ALTER TABLE JOB_HISTORY_RESULT ADD CONSTRAINT JOB_HIST_RESULT_JOB_HIST_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);


CREATE TABLE JOB_CHAIN (  
	JOB_CHAIN_ID     	NUMBER(10)                      PRIMARY KEY,  
	SOURCE_JOB_ID		NUMBER(10) 						NOT NULL,  
	CHAINED_TO_JOB_ID	NUMBER(10) 						NOT NULL,  
	CHAINING_ENABLED	NUMBER(1) default '1'			NOT NULL,  
	CHAINING_SCHEDULE        NVARCHAR2(255)					NULL,  
	REVISION            NUMBER(10)  default '0'         NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    	                NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)    	            NOT NULL
);
    
CREATE INDEX JOB_CHAIN_SOURCE_JOB_ID_NU ON JOB_CHAIN (SOURCE_JOB_ID);
CREATE INDEX JOB_CHAIN_CHAINED_TO_JOB_ID_NU ON JOB_CHAIN (CHAINED_TO_JOB_ID);
CREATE INDEX JOB_CHAIN_BOTH_JOB_IDS_NU ON JOB_CHAIN (SOURCE_JOB_ID, CHAINED_TO_JOB_ID);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_SOURCE_JOB_FK FOREIGN KEY (SOURCE_JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_CHAINED_TO_FK FOREIGN KEY (CHAINED_TO_JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE JOB_CHAIN_MODE (  
	JOB_CHAIN_MODE_ID   NUMBER(10)                    	PRIMARY KEY,  
	JOB_CHAIN_ID     	NUMBER(10)                      NOT NULL,  
	STATUS      		NVARCHAR2(20)					NOT NULL,
	REVISION            NUMBER(10)  default '0'         NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    	                NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)    	            NOT NULL,  
	CONSTRAINT CHK_JOB_CHAIN_MODE_STATUS CHECK (STATUS IN ('FAILED','DIED','KILLED','MISSED','COMPLETED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONDITIONAL'))
);
    
CREATE INDEX JOB_CHAIN_MODE_JOB_CHAIN_ID_NU ON JOB_CHAIN_MODE (JOB_CHAIN_ID);
ALTER TABLE JOB_CHAIN_MODE ADD CONSTRAINT JOB_CHAIN_MODE_CHAIN_FK FOREIGN KEY (JOB_CHAIN_ID) REFERENCES JOB_CHAIN (JOB_CHAIN_ID);

CREATE TABLE JOB_CHAIN_MODE_CONDITION (  
	JOB_CHAIN_MODE_CONDITION_ID   NUMBER(10)			PRIMARY KEY,  
	JOB_CHAIN_MODE_ID     	NUMBER(10)                  NOT NULL,  
	CONTEXT_KEY			NVARCHAR2(255)					NOT NULL,
	OPERATOR			NVARCHAR2(30)					NOT NULL,
	VALUE				NVARCHAR2(2000)					    NULL,
	REVISION            NUMBER(10)  default '0'         NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    	                NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)    	            NOT NULL,  
	CONSTRAINT CHK_JOB_CHAIN_MD_COND_OP CHECK (OPERATOR IN ('EQUALS','NOT_EQUALS','IN','NOT_IN','GREATER_THAN','LESS_THAN','GREATER_THAN_OR_EQUAL','LESS_THAN_OR_EQUAL','STARTS_WITH','ENDS_WITH','CONTAINS','EXISTS','NOT_EXISTS','REGEXP'))
);

CREATE INDEX JOB_CH_MD_COND_JB_CH_MD_ID_NU ON JOB_CHAIN_MODE_CONDITION (JOB_CHAIN_MODE_ID);
ALTER TABLE JOB_CHAIN_MODE_CONDITION ADD CONSTRAINT JOB_CHAIN_MODE_CONDITION_FK FOREIGN KEY (JOB_CHAIN_MODE_ID) REFERENCES JOB_CHAIN_MODE (JOB_CHAIN_MODE_ID);

CREATE TABLE JOB_HISTORY_CHAIN (  
	JOB_HISTORY_CHAIN_ID    NUMBER(10)                    PRIMARY KEY,
	JOB_HISTORY_ID     		NUMBER(10)                    NOT NULL,  
	JOB_CHAIN_ID			NUMBER(10)					  NOT NULL,
	CHAIN_TRIGGER			NUMBER(1)					  NOT NULL,
	CHAIN_DETAIL			NVARCHAR2(255)					  NULL,
	REVISION            NUMBER(10)  default '0'           NOT NULL,  
	CREATED_DATE        TIMESTAMP                         NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                       NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                           NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                      NOT NULL
);

CREATE INDEX JOB_HIST_CHAIN_JOB_HIST_ID_NU ON JOB_HISTORY_CHAIN (JOB_HISTORY_ID, CHAIN_TRIGGER);
CREATE INDEX JOB_HIST_CHAIN_JOB_CHAIN_ID_NU ON JOB_HISTORY_CHAIN (JOB_CHAIN_ID);
CREATE UNIQUE INDEX JOB_HIST_JOB_HIST_CHAIN_UK_01 ON JOB_HISTORY_CHAIN (JOB_HISTORY_ID, JOB_CHAIN_ID);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HIST_CHAIN_JOB_HIST_FK FOREIGN KEY (JOB_HISTORY_ID) REFERENCES JOB_HISTORY (JOB_HISTORY_ID);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HIST_CHAIN_JOB_CHAIN_FK FOREIGN KEY (JOB_CHAIN_ID) REFERENCES JOB_CHAIN (JOB_CHAIN_ID);

CREATE TABLE JOB_CONFLICT_CONFIG (  
	JOB_CONFLICT_CONFIG_ID     	NUMBER(10)              PRIMARY KEY,  
	PRIORITY_JOB_ID				NUMBER(10) 				NOT NULL,  
	YIELDING_JOB_ID				NUMBER(10) 				NOT NULL,
	GROUP_NUMBER				NUMBER(10) 				NOT NULL,
	ORDINAL 					NUMBER(10) 				NOT NULL,
	REVISION            NUMBER(10)  default '0'         NOT NULL,  
	CREATED_DATE        TIMESTAMP                       NOT NULL,  
	CREATED_BY        NVARCHAR2(50)                     NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                         NOT NULL,  
	LAST_UPDATED_BY    NVARCHAR2(50)                    NOT NULL
);
    
CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_01 ON JOB_CONFLICT_CONFIG (PRIORITY_JOB_ID);
CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_02 ON JOB_CONFLICT_CONFIG (YIELDING_JOB_ID);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_PRIORITY_FK FOREIGN KEY (PRIORITY_JOB_ID) REFERENCES JOB (JOB_ID);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_YIELDING_FK FOREIGN KEY (YIELDING_JOB_ID) REFERENCES JOB (JOB_ID);

CREATE TABLE LICENCE (  
	LICENCE_ID        NUMBER(10)      		PRIMARY KEY,  
	LICENCE_TYPE      NVARCHAR2(20) 		NOT NULL,  
	ISSUE_DATE        TIMESTAMP             NOT NULL,
	EXPIRY_DATE       TIMESTAMP             NOT NULL,
	LICENCE_KEY       NVARCHAR2(255) 		NOT NULL,
	PROXY_RENEWAL_TIME      TIMESTAMP       	NULL,
	PROXY_LEASE_EXPIRY_TIME TIMESTAMP           NULL,
	PROXY_RENEWAL_IP        NVARCHAR2(255)		NULL,
	PROXY_RENEWAL_TOKEN     NVARCHAR2(255)		NULL,
	PURCHASE_DETAIL_MESSAGE NCLOB				NULL,
	REVISION         NUMBER(10) default '0' NOT NULL,  
	CREATED_DATE      TIMESTAMP             NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP             NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL,  
	CONSTRAINT CHK_LICENCE_TYPE CHECK (LICENCE_TYPE IN ('TRIAL','FLOAT'))
);

CREATE UNIQUE INDEX LICENCE_KEY_UK ON LICENCE (LICENCE_KEY);

CREATE TABLE KEY_INSTANCE (  
	KEY_INSTANCE_ID   NUMBER(10)                        PRIMARY KEY,
	LICENCE_ID        NUMBER(10)                        NOT NULL,  
	RENEWAL_TIME      TIMESTAMP                             NULL,
	LEASE_EXPIRY_TIME TIMESTAMP                              NULL,
	RENEWAL_IP        NVARCHAR2(255)						 NULL,
	RENEWAL_TOKEN     NVARCHAR2(255)						 NULL,
	EXPIRY_TIME 	  TIMESTAMP                              NULL,
	REVISION         NUMBER(10) default '0' 			 NOT NULL,  
	CREATED_DATE      TIMESTAMP                          NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    					 NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                          NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)   					 NOT NULL    
);

CREATE INDEX KEY_INSTANCE_LICENCE_ID_NU ON KEY_INSTANCE (LICENCE_ID);
ALTER TABLE KEY_INSTANCE ADD CONSTRAINT KEY_INSTANCE_LICENCE_FK FOREIGN KEY (LICENCE_ID) REFERENCES LICENCE (LICENCE_ID);

CREATE TABLE TRIAL_LICENCE (
	TRIAL_LICENCE_ID 	NUMBER(10)				PRIMARY KEY,
	IP_ADDRESS			NVARCHAR2(255)			NOT NULL,
	LICENCE_ID 		  NUMBER(10)				NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,  
	CREATED_DATE      TIMESTAMP                 NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    			NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP                 NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)   			NOT NULL
);

CREATE INDEX TRIAL_LICENCE_NU ON TRIAL_LICENCE (IP_ADDRESS);
ALTER TABLE TRIAL_LICENCE ADD CONSTRAINT TRIAL_LICENCE_LICENCE_FK FOREIGN KEY (LICENCE_ID) REFERENCES LICENCE (LICENCE_ID);

CREATE TABLE EVENT_LOG (
	EVENT_LOG_ID	NUMBER(10)				PRIMARY KEY,
	EVENT_LEVEL			NVARCHAR2(10)			NOT NULL,
	EVENT_TIME		TIMESTAMP				NOT NULL,
	HOST			NVARCHAR2(100)			NOT NULL,
	CATEGORY		NVARCHAR2(50)			NOT NULL,
	SUMMARY			NVARCHAR2(255)			NOT NULL,
	MESSAGE			NCLOB					    NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,   
	CREATED_DATE      TIMESTAMP             NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP             NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL
);

CREATE INDEX KEY_LOG_NU_01 ON EVENT_LOG (CATEGORY);
CREATE INDEX KEY_LOG_NU_02 ON EVENT_LOG (HOST);
CREATE INDEX KEY_LOG_NU_03 ON EVENT_LOG (EVENT_TIME, EVENT_LEVEL, CATEGORY, HOST);

CREATE TABLE SEMAPHORE (
	SEMAPHORE_ID	NUMBER(10)					PRIMARY KEY,
	NAME			NVARCHAR2(50)							NOT NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,    
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL
);

CREATE UNIQUE INDEX KEY_SEMAPHORE_UK_01 ON SEMAPHORE (NAME);

CREATE TABLE OPERATIONS_PARAMETER (
	OPERATIONS_PARAMETER_ID	NUMBER(10)					PRIMARY KEY,
	NAME				NVARCHAR2(100)							NOT NULL,
	VALUE				NVARCHAR2(255)						NOT NULL,
	TYPE				NVARCHAR2(100)							NULL,
	CATEGORY			NVARCHAR2(100)						NOT NULL,
	DESCRIPTION			NVARCHAR2(500)							NULL,
	EDITABLE			NUMBER(1)									NOT NULL,
	UPDATE_TIME			TIMESTAMP							NOT NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL
);

CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_01 ON OPERATIONS_PARAMETER (NAME);
CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_02 ON OPERATIONS_PARAMETER (CATEGORY);


CREATE TABLE EVENT_SUBSCRIBER (
	EVENT_SUBSCRIBER_ID	NUMBER(10)	PRIMARY KEY,
	ADDRESS			  NVARCHAR2(255)			NOT NULL,
	ADDRESS_TYPE      NVARCHAR2(20) 		NOT NULL, 
	ACTIVE 			  NUMBER(1) default '1' NOT NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL, 
	CONSTRAINT CHK_EVENT_SUB_ADDRESS_TYPE CHECK (ADDRESS_TYPE IN ('EMAIL'))  
);

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIBER_UK_01 ON EVENT_SUBSCRIBER (ADDRESS, ADDRESS_TYPE);

CREATE TABLE EVENT_SUBSCRIPTION (
	EVENT_SUBSCRIPTION_ID NUMBER(10)  PRIMARY KEY,
	EVENT_SUBSCRIBER_ID	NUMBER(10)    NOT NULL ,
	CATEGORY		  NVARCHAR2(255)			NOT NULL,
	ENTITY_ID		  NUMBER(10)			NULL,
	MIN_LEVEL		  NVARCHAR2(255)			NOT NULL,
	ACTIVE 			  NUMBER(1) default '1'		NOT NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL
);

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIPTION_UK_01 ON EVENT_SUBSCRIPTION (EVENT_SUBSCRIBER_ID, CATEGORY, ENTITY_ID);
CREATE INDEX KEY_EVENT_SUBSCRIBER_NU_01 ON EVENT_SUBSCRIPTION (CATEGORY);
ALTER TABLE EVENT_SUBSCRIPTION ADD CONSTRAINT EVENT_SUB_SUBSCRIBER_FK FOREIGN KEY (EVENT_SUBSCRIBER_ID) REFERENCES EVENT_SUBSCRIBER (EVENT_SUBSCRIBER_ID);

CREATE TABLE NOTIFICATION (
	NOTIFICATION_ID	NUMBER(10)		PRIMARY KEY,
	EVENT_LOG_ID	NUMBER(10)		NOT NULL,
	EVENT_SUBSCRIBER_ID	NUMBER(10)	NOT NULL,
	REVISION          NUMBER(10)   default '0'  NOT NULL,
	CREATED_DATE      TIMESTAMP              NOT NULL,  
	CREATED_BY        NVARCHAR2(50)    		NOT NULL,  
	LAST_UPDATED_DATE TIMESTAMP              NOT NULL,  
	LAST_UPDATED_BY   NVARCHAR2(50)    		NOT NULL 
);

CREATE UNIQUE INDEX KEY_NOTIFICATION_UK_01 ON NOTIFICATION (EVENT_LOG_ID, EVENT_SUBSCRIBER_ID);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_SUBSCRIBER_FK FOREIGN KEY (EVENT_SUBSCRIBER_ID) REFERENCES EVENT_SUBSCRIBER (EVENT_SUBSCRIBER_ID);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_LOG_FK FOREIGN KEY (EVENT_LOG_ID) REFERENCES EVENT_LOG (EVENT_LOG_ID);
