CREATE TABLE AUDIT_TRAIL (
  AUDIT_TRAIL_ID     NUMBER(10)                    	   primary key,
  TABLE_NAME        NVARCHAR2(50)					   NOT NULL,
  COLUMN_NAME       NVARCHAR2(50) 				       NOT NULL,
  ROW_ID            NUMBER(10)                         NOT NULL,
  ACTION            CHAR(1)							   NOT NULL,
  VALUE           NVARCHAR2(255)                       	   NULL,
  ENTITY_NAME        NVARCHAR2(50)                     NOT NULL,
  AUDIT_DATE         TIMESTAMP                         NOT NULL,
  REVISION            NUMBER(10)  default '0'          NOT NULL,
  CREATED_DATE        TIMESTAMP                        NOT NULL,
  CREATED_BY        NVARCHAR2(50)                      NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                          NOT NULL,
  LAST_UPDATED_BY    NVARCHAR2(50)                     NOT NULL,
  CONSTRAINT  CHK_ACTION_ENUM CHECK (ACTION IN ('I','U','D','C')) 
);

CREATE INDEX NU_AUDIT_TRAIL_01 ON AUDIT_TRAIL (TABLE_NAME, COLUMN_NAME);
CREATE INDEX NU_AUDIT_TRAIL_02 ON AUDIT_TRAIL (TABLE_NAME, ROW_ID);


CREATE TABLE ATTACHMENT (
  ATTACHMENT_ID     NUMBER(10)                        primary key,
  OWNER_ID            NUMBER(10)                           	 NULL,
  NAME                NVARCHAR2(500)                      NOT NULL,
  CONTENT_TYPE        NVARCHAR2(100)                      NOT NULL,
  ADDED_DATE        TIMESTAMP                            NOT NULL,
  CONFIRMED            NUMBER(1)                         NOT NULL,
  CONTENT            BLOB                                NULL,
  REVISION            NUMBER(10)                         NOT NULL,
  CREATED_DATE        TIMESTAMP                          NOT NULL,
  CREATED_BY        NVARCHAR2(50)                         NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                            NOT NULL,
  LAST_UPDATED_BY    NVARCHAR2(50)                        NOT NULL
);

CREATE TABLE DOMAIN_ELEMENT (
  DOMAIN_ELEMENT_ID NUMBER(10)                        primary key,
  DOMAIN            NVARCHAR2(50)                         NOT NULL,
  REF_CODE            NVARCHAR2(50)                       NOT NULL,
  DESCRIPTION        NVARCHAR2(200)                       NOT NULL,
  REVISION            NUMBER(10)                         NOT NULL ,
  CREATED_DATE        TIMESTAMP                          NOT NULL,
  CREATED_BY        NVARCHAR2(50)                         NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                            NOT NULL,
  LAST_UPDATED_BY    NVARCHAR2(50)                        NOT NULL
);
  
CREATE UNIQUE INDEX UK_DOMAIN_ELEMENT_01 ON DOMAIN_ELEMENT (DOMAIN, REF_CODE);


CREATE TABLE SEQUENCE_MANAGER (
  NAME                 NVARCHAR2(50)                  primary key,
  VALUE             NUMBER(10)                          NOT NULL,
  REVISION            NUMBER(10)                        NOT NULL,
  CREATED_DATE        TIMESTAMP                         NOT NULL,
  CREATED_BY        NVARCHAR2(50)                        NOT NULL,
  LAST_UPDATED_DATE TIMESTAMP                           NOT NULL,
  LAST_UPDATED_BY    NVARCHAR2(50)                       NOT NULL
);


CREATE TABLE SUITE_USER (
 SUITE_USER_ID        NUMBER(10)                    primary key,
 USER_NAME            NVARCHAR2(50)                     NOT NULL,
 PASSWORD            NVARCHAR2(100)                     NOT NULL,
 FIRST_NAME            NVARCHAR2(50)                    NULL,
 LAST_NAME            NVARCHAR2(50)                     NULL,
 EMAIL                NVARCHAR2(50)                     NULL,
 ACTIVE        		   NUMBER(1)  default '1'		   NOT NULL, 
 REVISION            NUMBER(10)                    	   NOT NULL,
 CREATED_DATE        TIMESTAMP                         NOT NULL,
 CREATED_BY            NVARCHAR2(50)                   NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                          NOT NULL,
 LAST_UPDATED_BY    NVARCHAR2(50)               	   NOT NULL
);

CREATE UNIQUE INDEX UK_USER_USER_NAME ON SUITE_USER (USER_NAME);

CREATE TABLE USER_COOKIE (
 USER_COOKIE_ID            NUMBER(10)  		primary key,
 USER_NAME            NVARCHAR2(50)             NOT NULL,
 COOKIE            	  NVARCHAR2(255)            NOT NULL, 
 USER_STATE           NCLOB                     NOT NULL, 
 REVISION            NUMBER(10)                NOT NULL,
 CREATED_DATE        TIMESTAMP                 NOT NULL,
 CREATED_BY            NVARCHAR2(50)            NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                  NOT NULL,
 LAST_UPDATED_BY    NVARCHAR2(50)               NOT NULL
);

CREATE UNIQUE INDEX UK_USER_COOKIE_USER_NAME ON USER_COOKIE (USER_NAME);
CREATE UNIQUE INDEX UK_USER_COOKIE_COOKIE ON USER_COOKIE (COOKIE);


CREATE TABLE ROLE (
 ROLE_ID            NUMBER(10)                    primary key,
 ROLE_NAME            NVARCHAR2(50)                   NOT NULL,
 REVISION            NUMBER(10)                      NOT NULL ,
 CREATED_DATE        TIMESTAMP                       NOT NULL,
 CREATED_BY            NVARCHAR2(50)                  NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                        NOT NULL,
 LAST_UPDATED_BY    NVARCHAR2(50)                     NOT NULL
);

CREATE UNIQUE INDEX UK_ROLE_NAME ON ROLE (ROLE_NAME);

CREATE TABLE USER_ROLE (
 USER_ROLE_ID        NUMBER(10)                    primary key,
 SUITE_USER_ID		NUMBER(10)                        NOT NULL,
 ROLE_ID            NUMBER(10)                        NOT NULL,
 REVISION            NUMBER(10)                       NOT NULL ,
 CREATED_DATE        TIMESTAMP                        NOT NULL,
 CREATED_BY            NVARCHAR2(50)                   NOT NULL,
 LAST_UPDATED_DATE  TIMESTAMP                         NOT NULL,
 LAST_UPDATED_BY    NVARCHAR2(50)                      NOT NULL,
 CONSTRAINT FK_USER_USER_ROLE FOREIGN KEY (SUITE_USER_ID) REFERENCES SUITE_USER (SUITE_USER_ID),
 CONSTRAINT FK_USER_ROLE FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID) 
);

CREATE UNIQUE INDEX UK_USER_ROLE ON USER_ROLE (SUITE_USER_ID, ROLE_ID);


CREATE TABLE ROLE_ACTION_MAP (
 ROLE_ACTION_MAP_ID                 NUMBER(10)                    primary key,
 ROLE_ID                            NUMBER(10)                   	 NOT NULL,
 SERVLET_CLASS                        NVARCHAR2(100)                  NOT NULL,
 ACTION_NAME                        NVARCHAR2(50)                     NOT NULL,
 REVISION                            NUMBER(10)                      NOT NULL,
 CREATED_DATE                        TIMESTAMP                       NOT NULL,
 CREATED_BY                            NVARCHAR2(50)                  NOT NULL,
 LAST_UPDATED_DATE                  TIMESTAMP                        NOT NULL,
 LAST_UPDATED_BY                    NVARCHAR2(50)                     NOT NULL,
 CONSTRAINT FK_ROLE_ACTION_MAP_ROLE FOREIGN KEY (ROLE_ID) REFERENCES ROLE (ROLE_ID)
);

CREATE UNIQUE INDEX UK_ROLE_ACTION_MAP_01 ON ROLE_ACTION_MAP (ROLE_ID,SERVLET_CLASS,ACTION_NAME);
