CREATE TABLE JOB (
	job_id integer PRIMARY KEY,
	job_class character varying(255) NOT NULL,
	nickname character varying(50) NOT NULL,
	pickup_buffer_minutes integer NOT NULL,
	recovery_type character varying(10) NOT NULL,
	min_execution_duration character varying(15) NULL,
	max_execution_duration character varying(15) NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB ADD CHECK (recovery_type in ('NONE','ALL','LAST','CONFLICTED'));
CREATE UNIQUE INDEX JOB_NICKNAME_UK ON JOB USING btree (NICKNAME);

CREATE TABLE JOB_STATE (
	job_state_id integer PRIMARY KEY,
	job_id integer NOT NULL,
	schedule character varying(255),
	job_status character varying(50) NOT NULL,
	effective_date timestamp NOT NULL,
	end_date timestamp NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB_STATE ADD CHECK (job_status in ('DISABLED','ENABLED','CHAIN ACTIVE','AD HOC ACTIVE','UNSCHEDULED ACTIVE'));
CREATE INDEX JOB_STATE_JOB_ID_NU ON JOB_STATE USING btree (job_id);
ALTER TABLE JOB_STATE ADD CONSTRAINT JOB_STATE_JOB_FK FOREIGN KEY (job_id) REFERENCES JOB (job_id) ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE TABLE JOB_RUNNING_HOST (
	job_running_host_id integer PRIMARY KEY,
	job_id integer NOT NULL,
	running_host character varying(50) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX JOB_RUNNING_HOST_NU_01 ON JOB_RUNNING_HOST USING btree (job_id);
CREATE UNIQUE INDEX JOB_RUNNING_HOST_HOST_UK ON JOB_RUNNING_HOST USING btree (job_id, RUNNING_HOST);
ALTER TABLE JOB_RUNNING_HOST ADD CONSTRAINT JOB_RUNNING_HOST_JOB_FK FOREIGN KEY (job_id) REFERENCES JOB (job_id) ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE TABLE JOB_CONFIGURATION (
	job_configuration_id integer PRIMARY KEY,
	job_id integer NOT NULL,
	parameter_name character varying(50) NOT NULL,
	parameter_type character varying(255) NOT NULL,
	value text NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX JOB_CONFIGURATION_JOB_ID_NU_01 ON JOB_CONFIGURATION USING btree (job_id);
ALTER TABLE JOB_CONFIGURATION ADD CONSTRAINT JOB_CONFIGURATION_JOB_FK FOREIGN KEY (job_id) REFERENCES JOB (job_id) ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE TABLE JOB_HISTORY (
	job_history_id integer PRIMARY KEY,
	job_id integer NOT NULL,
	status character varying(15) NOT NULL,
	scheduled_time timestamp NOT NULL,
	running_host character varying(50),
	pickup_time timestamp,
	heartbeat_time timestamp,
	end_time timestamp,
	ad_hoc boolean NOT NULL,
	resubmission boolean NOT NULL,
	chained_from_id integer,
	resubmitted_to_id integer,
	submission_mode character varying(20),
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB_HISTORY ADD CHECK (status in ('READY','RUNNING','COMPLETED','FAILED','DIED','CONFLICTED','MISSED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONFLICT MISSED'));
ALTER TABLE JOB_HISTORY ADD CHECK (submission_mode in ('CONFLICTED_RECOVERY', 'CHAIN_SUBMISSION'));

CREATE INDEX JOB_HISTORY_STATUS_NU ON JOB_HISTORY USING btree (status);
CREATE INDEX JOB_HISTORY_JOB_ID_NU ON JOB_HISTORY USING btree (job_id);
CREATE INDEX JOB_HISTORY_RUNNING_HOST_NU ON JOB_HISTORY USING btree (running_host);
CREATE INDEX JOB_HISTORY_CHAINED_FROM_JOB_HISTORY_ID_NU ON JOB_HISTORY USING btree (chained_from_id);
CREATE INDEX JOB_HISTORY_RESUBMITTED_TO_JOB_HISTORY_ID_NU ON JOB_HISTORY USING btree (resubmitted_to_id);
CREATE UNIQUE INDEX JOB_HISTORY_SCHEDULED_JOB_UK ON JOB_HISTORY USING btree (job_id, scheduled_time);
CREATE INDEX JOB_HISTORY_SCHEDULED_TIME_NU ON JOB_HISTORY USING btree (scheduled_time);

ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_JOB_FK FOREIGN KEY (job_id) REFERENCES JOB (job_id) ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_CHAINED_FROM_FK FOREIGN KEY (chained_from_id) REFERENCES JOB_HISTORY (job_history_id) ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE JOB_HISTORY ADD CONSTRAINT JOB_HISTORY_RESUBMITTED_TO_FK FOREIGN KEY (resubmitted_to_id) REFERENCES JOB_HISTORY (job_history_id) ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE TABLE JOB_HISTORY_ERROR (
	job_history_error_id integer PRIMARY KEY,
	job_history_id integer NOT NULL,
	exception_class character varying(300) NOT NULL,
	exception_message character varying(255) NOT NULL,
	error text NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX JOB_HISTORY_ERROR_JOB_HISTORY_ID_UK_01 ON JOB_HISTORY_ERROR USING btree (job_history_id);
ALTER TABLE JOB_HISTORY_ERROR ADD CONSTRAINT JOB_HISTORY_ERROR_JOB_HISTORY_FK FOREIGN KEY (job_history_id) REFERENCES JOB_HISTORY (job_history_id);

CREATE TABLE JOB_HISTORY_INTERRUPT (
	job_history_interrupt_id integer PRIMARY KEY,
	job_history_id integer NOT NULL,
	requester character varying(50) NOT NULL,
	request_time timestamp NOT NULL,
	interrupt_time timestamp NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);
CREATE UNIQUE INDEX JOB_HISTORY_INTERRUPT_JOB_HISTORY_ID_UK_01 ON JOB_HISTORY_INTERRUPT USING btree (job_history_id);
ALTER TABLE JOB_HISTORY_INTERRUPT ADD CONSTRAINT JOB_HISTORY_INTERRUPT_JOB_HISTORY_FK FOREIGN KEY (job_history_id) REFERENCES JOB_HISTORY (job_history_id);

CREATE TABLE JOB_HISTORY_RESULT (
	job_history_result_id integer PRIMARY KEY,
	job_history_id integer NOT NULL,
	name character varying(255) NOT NULL,
	value text,
	value_type character varying(255) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB_HISTORY_RESULT ADD CONSTRAINT JOB_HISTORY_RESULT_JOB_HISTORY_FK FOREIGN KEY (job_history_id) REFERENCES JOB_HISTORY (job_history_id);

CREATE TABLE JOB_CHAIN (
	job_chain_id integer PRIMARY KEY,
	source_job_id integer NOT NULL,
	chained_to_job_id integer NOT NULL,
	chaining_enabled boolean NOT NULL,
	chaining_schedule character varying(255),
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX JOB_CHAIN_SOURCE_JOB_ID_NU ON JOB_CHAIN USING btree (source_job_id);
CREATE INDEX JOB_CHAIN_CHAINED_TO_JOB_ID_NU ON JOB_CHAIN USING btree (chained_to_job_id);
CREATE INDEX JOB_CHAIN_BOTH_JOB_IDS_NU ON JOB_CHAIN (source_job_id, chained_to_job_id);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_SOURCE_JOB_FK FOREIGN KEY (source_job_id) REFERENCES JOB (job_id);
ALTER TABLE JOB_CHAIN ADD CONSTRAINT JOB_CHAIN_CHAINED_TO_FK FOREIGN KEY (chained_to_job_id) REFERENCES JOB (job_id);

CREATE TABLE JOB_CHAIN_MODE (
	job_chain_mode_id integer PRIMARY KEY,
	job_chain_id integer NOT NULL,
	status character varying(13) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB_CHAIN_MODE ADD CONSTRAINT job_chain_mode_status_check CHECK (status in ('FAILED','DIED','KILLED','MISSED','COMPLETED','ABANDONED','OVERLAPPED','CHAIN SKIPPED','CONDITIONAL'));
CREATE INDEX JOB_CHAIN_MODE_JOB_CHAIN_ID_NU ON JOB_CHAIN_MODE USING btree (job_chain_id);
ALTER TABLE JOB_CHAIN_MODE ADD CONSTRAINT JOB_CHAIN_MODE_JOB_CHAIN_ID_FK FOREIGN KEY (job_chain_id) REFERENCES JOB_CHAIN (job_chain_id);

CREATE TABLE JOB_CHAIN_MODE_CONDITION (
	job_chain_mode_condition_id integer PRIMARY KEY,
	job_chain_mode_id integer NOT NULL,
	context_key		character varying(255) NOT NULL,
	operator			character varying(30) NOT NULL,
	value				text NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE JOB_CHAIN_MODE_CONDITION ADD CHECK (operator in ('EQUALS','NOT_EQUALS','IN','NOT_IN','GREATER_THAN','LESS_THAN','GREATER_THAN_OR_EQUAL','LESS_THAN_OR_EQUAL','STARTS_WITH','ENDS_WITH','CONTAINS','EXISTS','NOT_EXISTS','REGEXP'));
CREATE INDEX JOB_CHAIN_MODE_CONDITION_JOB_CHAIN_MODE_ID_NU ON JOB_CHAIN_MODE_CONDITION USING btree (job_chain_mode_id);
ALTER TABLE JOB_CHAIN_MODE_CONDITION ADD CONSTRAINT JOB_CHAIN_MODE_CONDITION_FK FOREIGN KEY (job_chain_mode_id) REFERENCES JOB_CHAIN_MODE (job_chain_mode_id);


CREATE TABLE JOB_HISTORY_CHAIN (
	job_history_chain_id integer PRIMARY KEY,
	job_history_id integer NOT NULL,
	job_chain_id integer NOT NULL,
	chain_trigger boolean NOT NULL,
	chain_detail character varying(255),
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX JOB_HISTORY_CHAIN_JOB_HISTORY_ID_NU ON JOB_HISTORY_CHAIN USING btree (job_history_id, chain_trigger);
CREATE INDEX JOB_HISTORY_CHAIN_JOB_CHAIN_ID_NU ON JOB_HISTORY_CHAIN USING btree (job_chain_id);
CREATE UNIQUE INDEX JOB_HISTORY_JOB_HISTORY_CHAIN_UK_01 ON JOB_HISTORY_CHAIN USING btree (job_history_id, job_chain_id);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HISTORY_CHAIN_JOB_HISTORY_FK FOREIGN KEY (job_history_id) REFERENCES JOB_HISTORY (job_history_id);
ALTER TABLE JOB_HISTORY_CHAIN ADD CONSTRAINT JOB_HISTORY_CHAIN_JOB_CHAIN_FK FOREIGN KEY (job_chain_id) REFERENCES JOB_CHAIN (job_chain_id);

CREATE TABLE JOB_CONFLICT_CONFIG (
	job_conflict_config_id integer PRIMARY KEY,
	priority_job_id integer NOT NULL,
	yielding_job_id integer NOT NULL,
	group_number integer NOT NULL,
	ordinal integer NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_01 ON JOB_CONFLICT_CONFIG USING btree (priority_job_id);
CREATE UNIQUE INDEX JOB_CONFLICT_CONFIG_UK_02 ON JOB_CONFLICT_CONFIG USING btree (yielding_job_id);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_CONFIG_PRIORITY_JOB_FK FOREIGN KEY (priority_job_id) REFERENCES JOB (job_id);
ALTER TABLE JOB_CONFLICT_CONFIG ADD CONSTRAINT JOB_CONFLICT_CONFIG_YIELDING_JOB_FK FOREIGN KEY (yielding_job_id) REFERENCES JOB (job_id);


CREATE TABLE LICENCE (
	licence_id integer PRIMARY KEY,
	licence_type character varying(5) NOT NULL,
	issue_date timestamp NOT NULL,
	expiry_date timestamp NOT NULL,
	licence_key character varying(255) NOT NULL,
	proxy_renewal_time timestamp,
	proxy_lease_expiry_time timestamp,
	proxy_renewal_ip character varying(255),
	proxy_renewal_token character varying(255),
	purchase_detail_message text,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX LICENCE_KEY_UK ON LICENCE USING btree (licence_key);

CREATE TABLE KEY_INSTANCE (
	key_instance_id integer PRIMARY KEY,
	licence_id integer NOT NULL,
	renewal_time timestamp,
	lease_expiry_time timestamp,
	renewal_ip character varying(255),
	renewal_token character varying(255),
	expiry_time timestamp,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX KEY_INSTANCE_LICENCE_ID_NU ON KEY_INSTANCE USING btree (licence_id);
ALTER TABLE KEY_INSTANCE ADD CONSTRAINT KEY_INSTANCE_LICENCE_FK FOREIGN KEY (licence_id) REFERENCES LICENCE (licence_id);

CREATE TABLE TRIAL_LICENCE (
	trial_licence_id integer PRIMARY KEY,
	ip_address character varying(255),
	licence_id integer,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE TRIAL_LICENCE ADD CONSTRAINT TRIAL_LICENCE_LICENCE_FK FOREIGN KEY (licence_id) REFERENCES LICENCE (licence_id);
CREATE INDEX TRIAL_LICENCE_NU ON TRIAL_LICENCE USING btree (ip_address);

CREATE TABLE EVENT_LOG (
	event_log_id integer PRIMARY KEY,
	event_level character varying(10) NOT NULL,
	event_time timestamp NOT NULL,
	host character varying(100) NOT NULL,
	category character varying(50) NOT NULL,
	summary character varying(255) NOT NULL,
	message text,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX KEY_LOG_NU_01 ON EVENT_LOG USING btree (category);
CREATE INDEX KEY_LOG_NU_02 ON EVENT_LOG USING btree (host);
CREATE INDEX KEY_LOG_NU_03 ON EVENT_LOG USING btree (event_time, event_level, category, host);

CREATE TABLE SEMAPHORE (
	semaphore_id integer PRIMARY KEY,
	name character varying(50) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX KEY_SEMAPHORE_UK_01 ON SEMAPHORE USING btree (name);

CREATE TABLE OPERATIONS_PARAMETER (
	operations_parameter_id integer PRIMARY KEY,
	name character varying(100) NOT NULL,
	value character varying(255) NOT NULL,
	type character varying(100),
	category character varying(100) NOT NULL,
	description character varying(500),
	editable boolean NOT NULL,
	update_time timestamp NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_01 ON OPERATIONS_PARAMETER USING btree (name);
CREATE INDEX KEY_OPERATIONS_PARAMETER_NU_02 ON OPERATIONS_PARAMETER USING btree (category);

CREATE TABLE EVENT_SUBSCRIBER (
	event_subscriber_id integer PRIMARY KEY,
	address character varying(255) NOT NULL,
	address_type character varying(5) NOT NULL,
	active boolean NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIBER_UK_01 ON EVENT_SUBSCRIBER USING btree (address, address_type);

CREATE TABLE EVENT_SUBSCRIPTION (
	event_subscription_id integer PRIMARY KEY,
	event_subscriber_id integer NOT NULL,
	category character varying(255) NOT NULL,
	entity_id integer,
	min_level character varying(255) NOT NULL,
	active boolean NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX KEY_EVENT_SUBSCRIBTION_UK_01 ON EVENT_SUBSCRIPTION USING btree (event_subscriber_id, category, entity_id);
CREATE INDEX KEY_EVENT_SUBSCRIBER_NU_01 ON EVENT_SUBSCRIPTION USING btree (category);
ALTER TABLE EVENT_SUBSCRIPTION ADD CONSTRAINT EVENT_SUBCRIPTION_SUBSCRIBER_FK FOREIGN KEY (event_subscriber_id) REFERENCES EVENT_SUBSCRIBER (event_subscriber_id);

CREATE TABLE NOTIFICATION (
	notification_id integer PRIMARY KEY,
	event_log_id integer NOT NULL,
	event_subscriber_id integer NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX KEY_NOTIFICATION_UK_01 ON NOTIFICATION USING btree (event_log_id, event_subscriber_id);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_SUBSCRIBER_FK FOREIGN KEY (event_subscriber_id) REFERENCES event_subscriber (event_subscriber_id);
ALTER TABLE NOTIFICATION ADD CONSTRAINT NOTIFICATION_LOG_FK FOREIGN KEY (event_log_id) REFERENCES event_log (event_log_id);

CREATE OR REPLACE FUNCTION subtractInterval (dt timestamp, intrval INTERVAL) RETURNS timestamp AS $$
BEGIN
  RETURN dt - intrval;
END;
$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION addInterval (dt timestamp, intrval INTERVAL) RETURNS timestamp AS $$
BEGIN
  RETURN dt + intrval;
END;
$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION getInterval (qty INTEGER, intervalType text) RETURNS INTERVAL AS $$
BEGIN
  RETURN (qty || ' ' || intervalType)::INTERVAL;
END;
$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;