CREATE TABLE AUDIT_TRAIL (
	audit_trail_id integer PRIMARY KEY,
	table_name character varying(50) NOT NULL,
	column_name character varying(50) NOT NULL,
	row_id integer NOT NULL,
	action character varying(1) NOT NULL,
	value character varying(255) NOT NULL,
	entity_name character varying(50) NOT NULL,
	audit_date timestamp NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL ,
	last_updated_by character varying(50) NOT NULL
);

ALTER TABLE AUDIT_TRAIL ADD CHECK (action in ('I','U','D','C'));
CREATE INDEX NU_AUDIT_TRAIL_01 ON AUDIT_TRAIL USING btree (table_name, column_name);
CREATE INDEX NU_AUDIT_TRAIL_02 ON AUDIT_TRAIL USING btree (table_name, row_id);

CREATE TABLE ATTACHMENT (
	attachment_id integer PRIMARY KEY,
	owner_id integer NOT NULL,
	name character varying(500) NOT NULL,
	content_type character varying(100) NOT NULL,
	added_date timestamp NOT NULL,
	confirmed boolean NOT NULL,
	content oid,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);


CREATE TABLE DOMAIN_ELEMENT (
	domain_element_id integer PRIMARY KEY,
	domain character varying(50) NOT NULL,
	ref_code character varying(50) NOT NULL,
	description character varying(200) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_DOMAIN_ELEMENT_01 ON DOMAIN_ELEMENT USING btree (domain, ref_code);

CREATE TABLE SEQUENCE_MANAGER (
	name character varying(50) PRIMARY KEY,
	value integer NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE TABLE SUITE_USER (
	suite_user_id integer PRIMARY KEY,
	user_name character varying(50) NOT NULL,
	password character varying(100) NOT NULL,
	first_name character varying(50)   NULL,
	last_name character varying(50)    NULL,
	email character varying(50)    NULL,
	active boolean NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_USER_USER_NAME ON SUITE_USER USING btree(user_name);

CREATE TABLE USER_COOKIE (
	user_cookie_id integer PRIMARY KEY,
	user_name character varying(50) NOT NULL,
	cookie character varying(255) NOT NULL,
	user_state text NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_USER_COOKIE_USER_NAME ON USER_COOKIE USING btree(user_name);
CREATE UNIQUE INDEX UK_USER_COOKIE_COOKIE ON USER_COOKIE USING btree(cookie);

CREATE TABLE ROLE (
	role_id integer PRIMARY KEY,
	role_name character varying(50) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_ROLE_NAME ON ROLE USING btree (role_name);

CREATE TABLE USER_ROLE (
	user_role_id integer PRIMARY KEY,
	suite_user_id integer NOT NULL,
	role_id integer NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_USER_ROLE ON USER_ROLE USING btree (suite_user_id, role_id);
CREATE INDEX FKI_USER_ROLE_USER ON USER_ROLE(suite_user_id);
ALTER TABLE USER_ROLE ADD CONSTRAINT FK_USER_ROLE_USER FOREIGN KEY (suite_user_id) REFERENCES SUITE_USER (suite_user_id) ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX FKI_USER_ROLE_ROLE ON USER_ROLE(role_id);
ALTER TABLE USER_ROLE ADD CONSTRAINT FK_USER_ROLE_ROLE FOREIGN KEY (role_id) REFERENCES ROLE (role_id) ON UPDATE NO ACTION ON DELETE NO ACTION;



CREATE TABLE ROLE_ACTION_MAP (
	role_action_map_id integer PRIMARY KEY,
	role_id integer NOT NULL,
	servlet_class character varying(100) NOT NULL,
	action_name character varying(50) NOT NULL,
	revision integer NOT NULL,
	created_date timestamp NOT NULL,
	created_by character varying(50) NOT NULL,
	last_updated_date timestamp NOT NULL,
	last_updated_by character varying(50) NOT NULL
);

CREATE UNIQUE INDEX UK_ROLE_ACTION_MAP_01 ON ROLE_ACTION_MAP USING btree (role_id,servlet_class,action_name);
CREATE INDEX FKI_ROLE_ACTION_MAP_ROLE ON ROLE_ACTION_MAP(role_id);
ALTER TABLE ROLE_ACTION_MAP ADD CONSTRAINT FK_ROLE_ACTION_MAP_ROLE FOREIGN KEY (role_id) REFERENCES ROLE (role_id) ON UPDATE NO ACTION ON DELETE NO ACTION;
